#include "Vect.h"

double Vect::get(size_t const& i) const{
	if(i<(*this).getDim())	return (*this).comp[i];
	else return 0;
}

double Vect::operator[](size_t const& i){
	return get(i);
}


void Vect::set(size_t const& i, double const& nb){
	if(i<(*this).getDim()) (*this).comp[i]=nb;
	else {
			(*this).setDim(i+1);
			(*this).comp[i]=nb;
	}
}
void Vect::set(vector<double> const& comp){
		(*this).comp=comp;
}

void Vect::set0(){
		for(size_t i=0;i<(*this).getDim();i++){
				(*this).set(i,0);
		}
}

size_t Vect::getDim() const{
	return comp.size();
}
void Vect::addDim(double const& d){
	comp.push_back(d);
}
void Vect::subDim(){
	comp.pop_back();
}

//!NEW
Vect Vect::cut(size_t const& first,size_t nb_comp){
		if(nb_comp==0)nb_comp=getDim()-first;//! a verifier !!
		Vect result(nb_comp);
		for(size_t i=0;i<nb_comp;i++){
				result.set(i,get(i+first));
		}
		return result;
}
//!---

Vect Vect::project(Vect const& v){
	Vect dir(v.normalize());
	return ((*this)*dir)*dir;
}

Vect Vect::reject(Vect const& v){
	Vect proj=project(v);
	return (*this)-proj;
}


bool Vect::compareFast(Vect const& v) const{
		if (v.getDim()!=(*this).getDim()) return false;
		for(size_t i=0;i<v.getDim();i++){
		if(v.get(i)!=(*this).get(i)) return false;//si la distance est plus grande que tol (tolerance)
	}
	return true;
}

bool Vect::compare(Vect const& v,double const& tol,bool const& dimInegal) const{
	bool egalDim = (v.getDim()==(*this).getDim());

	if (dimInegal==false && !egalDim) {
		return false;
	}

	size_t minDim = min(v.getDim(),(*this).getDim());//si dimInegal est true, on va jusqu'à la dim la plus petite
	//on a deja elimine le cas dim inegal et pas de tolérance sur le nb de dim
	for(size_t i=0;i<minDim;i++){
		if(abs(v.get(i)-(*this).get(i))>tol) return false;//si la distance est plus grande que tol (tolerance)
	}
	return true;
}
bool Vect::compare2(Vect const& v,double const& tolNorme) const{
		double distCarre = ((*this).sub(v)).normeCarree();
		if(distCarre <= (tolNorme*tolNorme)){
				return true;
		}
		return false;
}
Vect Vect::add(Vect const& v,bool const&  bigDim) const{
	Vect result;
	size_t dim,myDim,vDim;

	myDim=(*this).getDim();
	vDim=v.getDim();

	if(bigDim){
		dim = min(vDim,myDim);
		if(dim==myDim){//si myDim est le plus petit, on copie v dans result puis on complete
			result.comp=v.comp;
			for(size_t i=0;i<dim;i++){
				result.set(i,result.get(i)+(*this).get(i));
			}//complete le reste
		}
		else {
			result.comp=(*this).comp;
			for(size_t i=0;i<dim;i++){
				result.set(i,result.get(i)+v.get(i));
			}//complete le reste
		}
	}
	else {//not bigDim
		dim = min(v.getDim(),(*this).getDim());
		if(dim==myDim){
			result.comp=(*this).comp;
			for(size_t i=0;i<dim;i++){
				result.set(i,result.get(i)+v.get(i));
			}
		}
		else{
			result.comp=v.comp;
			for(size_t i=0;i<dim;i++){
				result.set(i,result.get(i)+(*this).get(i));
			}
		}
	}


	return result;
}
Vect Vect::sub(Vect const& v,bool const& bigDim) const{
	return (*this).add(v.oppose(),bigDim);
}
Vect Vect::oppose() const{
	Vect result;
	size_t dim = (*this).getDim();
	for(size_t i=0;i<dim;i++){
		result.addDim((-1)*(*this).get(i));
	}
	return result;
}
Vect Vect::mult(double const& d) const{
	Vect result;
	size_t dim=(*this).getDim();
	for(size_t i=0;i<dim;i++){
		result.addDim((*this).get(i) * d );
	}
	return result;
}
double Vect::prodScal(Vect const& v) const{
	double result=0;
	size_t dim= min( (*this).getDim() , v.getDim());
	for(size_t i=0;i<dim;i++){
		result += v.get(i) * (*this).get(i);
	}
	return result;
}
Vect Vect::prodVect(Vect const& v) const{
		Vect result;
		if(((*this).getDim()!=3)||(v.getDim()!=3)){
				result.addDim(0);
				result.addDim(0);
				result.addDim(0);
				return result;
		}
		result.addDim( (*this).get(1) * v.get(2) - (*this).get(2) * v.get(1) );
		result.addDim( (*this).get(2) * v.get(0) - (*this).get(0) * v.get(2) );
		result.addDim( (*this).get(0) * v.get(1) - (*this).get(1) * v.get(0) );

	return result;
}
Vect Vect::normalize() const{
		if(normeCarree()!=0)
		return Vect((*this)/(*this).norme());
		else return Vect((*this).getDim());
}
double Vect::norme() const{
	return sqrt( (*this).normeCarree() );
}
double Vect::normeCarree() const{
	double result=0;
	size_t dim=(*this).getDim();
	for(size_t i=0;i<dim;i++){
		result += pow( (*this).get(i) , 2);
	}
	return result;
}
void Vect::setDim(size_t const& dim){
	int change= (int)dim - (int)(*this).getDim();

	if(change==0) return;
	for(;change<0;change++){
			(*this).subDim();
	}
	for(;change>0;change--){
			(*this).addDim();
	}
}

//surcharge operator
Vect Vect::operator+(Vect const& v) const{
	return (*this).add(v);
}
Vect Vect::operator-(Vect const& v) const{
	return (*this).sub(v);
}
Vect Vect::operator-() const{
	return (*this).oppose();
}
Vect& Vect::operator+=(Vect const& v){
	return ( (*this)=((*this).add(v)) );
}
Vect& Vect::operator-=(Vect const& v){
	return ( (*this)=((*this).sub(v)) );
}
Vect Vect::operator^(Vect const& v) const{
	return (*this).prodVect(v);
}
Vect Vect::operator*(double const& nb) const{
	return (*this).mult(nb);
}

Vect Vect::operator/(double const& nb)const {
		return (*this).mult(1/nb);
}

double Vect::operator*(Vect const& v)const{
	return (*this).prodScal(v);
}
bool Vect::operator==(Vect const& v)const{
	return (*this).compareFast(v);
}
bool Vect::operator!=(Vect const& v)const{
		return !((*this)==v);
}
//double operator
double Vect::operator||(bool const& sqrt)const{
		if(sqrt) return (*this).norme();
		return (*this).normeCarree();
}

Vect& Vect::operator*=(double const& nb){
		return ((*this)=(*this)*nb);
}

Vect& Vect::operator/=(double const& nb){
		return ( (*this)=(*this)/nb );
}

Vect operator*(double const& a, Vect const& v){
		return v*a;
}
Vect operator*(int const& a, Vect const& v){
		return v*(double)a;
}
Vect operator*(unsigned int const& a, Vect const& v){
		return v*(double)a;
}


ostream& operator<<(ostream& out,Vect const& v){//testé
		size_t n= v.getDim();
	for(size_t i=0;i<n;i++){
		out<<v.get(i)<<" ";
	}
	out<<endl;
	return out;
}












//constructeurs
Vect::Vect(){}

Vect::Vect(size_t const& dim):comp(dim,0){}

Vect::Vect(initializer_list<double> const& v):comp(v){}//testé

Vect::Vect(vector<double> v):comp(v){}//testé


//Vect::Vect(double const& a,double const&b,double const&c):comp({a,b,c}){}//testé



















